/*********************************************************************************
 **Copyright (C) 2008 by Daniel Sudmann						                    **
 **										                                        **
 **This software is provided 'as-is', without any express or implied		    **
 **warranty.  In no event will the authors be held liable for any damages	    **
 **arising from the use of this software.					                    **
 **										                                        **
 **Permission is not granted to anyone to use this software for any purpose,	**
 **including private and commercial applications, and to alter it and		    **
 **redistribute it.								                                **
 **										                                        **
 *********************************************************************************/
#ifndef CATTRIBUTENODE_H
#define CATTRIBUTENODE_H

#include <irrlicht.h>

namespace irr
{
    namespace scene
    {
    	class CAttribute
    	{
		public:
			enum ATTRIBUTE_TYPE
			{
				ATTRIBUTE_INT = 0,
				ATTRIBUTE_FLOAT = 1,
				ATTRIBUTE_BOOL = 2,
				ATTRIBUTE_STRING = 3,
				ATTRIBUTE_VECTOR3D = 4
			};
			CAttribute() : Type(ATTRIBUTE_INT), Name("")
			{
				Int = 0;
				Float = 0;
				Bool = false;
			}
			CAttribute(ATTRIBUTE_TYPE type, irr::core::stringc name) : Type(type), Name(name)
			{
				Int = 0;
				Float = 0;
				Bool = false;
			}
			ATTRIBUTE_TYPE getType(void)
			{
				return Type;
			}
			const irr::core::stringc& getName(void)
			{
				return Name;
			}
			irr::s32& getInt(void){return Int;}
			irr::f32& getFloat(void){return Float;}
			bool& getBool(void){return Bool;}
			irr::core::stringc& getString(void){return String;}
			irr::core::vector3df& getVector3D(void){return Vector3D;}
			void clear(void)
			{
				delete this;
			}
		protected:
			ATTRIBUTE_TYPE Type;
			irr::core::stringc Name;
			irr::s32 Int;
			irr::f32 Float;
			bool Bool;
			irr::core::stringc String;
			irr::core::vector3df Vector3D;
    	};
        class CAttributeNode : public ISceneNode
        {
        public:
            CAttributeNode(ISceneNode* parent, IrrlichtDevice* device, s32 id = -1, core::vector3df pos = core::vector3df(0,0,0), core::vector3df rot = core::vector3df(0,0,0), core::vector3df scale = core::vector3df(1,1,1));
            virtual ~CAttributeNode();

            void deserializeAttributes(io::IAttributes *in, io::SAttributeReadWriteOptions *options=0);
            void serializeAttributes(io::IAttributes *out, io::SAttributeReadWriteOptions *options=0) const;

            irr::core::array<CAttribute*>& getAttributes(void);

            void setEntity(irr::core::stringc name);

			void OnRegisterSceneNode(void);

            void render(void){}

			ESCENE_NODE_TYPE getType() const;

			const irr::core::stringc& getEntityType(void);

			u32 getMaterialCount()
			{
				return 1;
			}

			video::SMaterial& getMaterial(u32 i)
			{
				return Material;
			}

            const irr::core::aabbox3d<irr::f32>& getBoundingBox(void) const
            {
            	return Box;
            }
        protected:
			irr::core::stringc EntityType;

			irr::core::array<CAttribute*> Attributes;

			void loadAttributes(const irr::c8* name);

			IrrlichtDevice* Device;

			irr::core::aabbox3d<irr::f32> Box;

			video::SMaterial Material;
        private:
        };
    }
}
#endif // CATTRIBUTENODE_H
