#include "EntitySceneNodePlugin.h"

// this Entity shows how to add own scene nodes into irrEdit.
// You only need two things:
// - a scene node factory 
// - your scene node.
// you can add as many scene nodes with a plugin as you wish.


// entry point for the plugin, will be called by the editor. Return a
// plugin interface if your plugin should run as plugin inside the editor, otherwise
// just modify the engine (add factories etc) and return 0.
__declspec(dllexport) irr::irredit::IIrrEditPlugin* __stdcall createPlugin( irr::irredit::IrrEditServices* services )
{
	// we simple register a new scene node factory here, which is only able to
	// create our Entity scene node
	
	scene::ISceneManager* mgr = services->getDevice()->getSceneManager();

	CEntitySceneNodeFactory* factory = new CEntitySceneNodeFactory(mgr, services->getDevice());
	mgr->registerSceneNodeFactory(factory);
	factory->drop();

	return 0;
}



// -------------------------------------------------------------------------------
// Entity factory implementation
// -------------------------------------------------------------------------------

const char* entitySceneNodeTypeName = "Entity";
const int ENTITY_SCENE_NODE_ID = MAKE_IRR_ID('e','n','t','i');

#include "CAttributeNode.h"

namespace irr
{
	namespace scene
	{
		ESCENE_NODE_TYPE CAttributeNode::getType() const
		{
			return (ESCENE_NODE_TYPE)ENTITY_SCENE_NODE_ID;
		}
	}
}

CEntitySceneNodeFactory::CEntitySceneNodeFactory(ISceneManager* mgr, IrrlichtDevice* device)
: Manager(mgr), Device(device)
{
	// don't grab the manager here, to avoid cyclic references
}


CEntitySceneNodeFactory::~CEntitySceneNodeFactory()
{
}


//! adds a scene node to the scene graph based on its type id
ISceneNode* CEntitySceneNodeFactory::addSceneNode(ESCENE_NODE_TYPE type, ISceneNode* parent)
{
	if (!parent)
		parent = Manager->getRootSceneNode();

	if (type == ENTITY_SCENE_NODE_ID)
	{
		CAttributeNode* node = new CAttributeNode(parent, Device, -1);
		node->drop();
		return node; 
	}

	return 0;
}


//! adds a scene node to the scene graph based on its type name
ISceneNode* CEntitySceneNodeFactory::addSceneNode(const c8* typeName, ISceneNode* parent)
{
	return addSceneNode( getTypeFromName(typeName), parent );
}


//! returns amount of scene node types this factory is able to create
u32 CEntitySceneNodeFactory::getCreatableSceneNodeTypeCount() const
{
	return 1;
}


//! returns type of a createable scene node type
ESCENE_NODE_TYPE CEntitySceneNodeFactory::getCreateableSceneNodeType(u32 idx) const
{
	if (idx==0)
		return (ESCENE_NODE_TYPE)ENTITY_SCENE_NODE_ID;

	return ESNT_UNKNOWN;
}


//! returns type name of a createable scene node type 
const c8* CEntitySceneNodeFactory::getCreateableSceneNodeTypeName(u32 idx) const
{
	if (idx==0)
		return entitySceneNodeTypeName;

	return 0;
}


//! returns type name of a createable scene node type 
const c8* CEntitySceneNodeFactory::getCreateableSceneNodeTypeName(ESCENE_NODE_TYPE type) const
{
	if (type == ENTITY_SCENE_NODE_ID)
		return entitySceneNodeTypeName;

	return 0;
}


ESCENE_NODE_TYPE CEntitySceneNodeFactory::getTypeFromName(const c8* name)
{
	if (!strcmp(name, entitySceneNodeTypeName))
		return (ESCENE_NODE_TYPE)ENTITY_SCENE_NODE_ID;

	return ESNT_UNKNOWN;
}
